<?php

namespace Drush\Commands\entity_dumper;

use Drush\Commands\DrushCommands;

/**
 * Entity dumper commandfile.
 */
class EntityDumperCommands extends DrushCommands {

  /**
   * Dumps an entity.
   *
   * @param $entity_type_id
   *   Entity type ID (for example node).
   * @param $entity_id
   *   The entity id.
   * @param array $options
   *   An associative array of options whose values come from cli, aliases, config, etc.
   * @option descend
   *   A comma separated list of entity type IDs to descend into. Defaults to
   *   paragraph.
   * @option max-depth
   *   Maximum descend depth. Avoids infinite loops. Defaults to 0 which means
   *   no limit.
   * @usage entity-dumper --descend=paragraph,media node 92920
   *   Create an entity dump of node 92920 descending into paragraphs and media.
   *
   * @command entity-dumper
   * @bootstrap full
   */
  public function dump(string $entity_type_id, $entity_id, $options = ['descend' => 'paragraph', 'max-depth' => 0]) {
    /** @var \Drupal\Core\Entity\ContentEntityInterface $entity */
    $entity = \Drupal::entityTypeManager()->getStorage($entity_type_id)->load($entity_id);
    $descend = explode(',', $options['descend']);
    include_once __DIR__ . '/EntityDumperHelper.php';
    $this->output()->write(EntityDumperHelper::dumpEntity($entity, $descend, $options['max-depth']));
  }

}
